@@ -1,6 +1,6 @@
 /* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
 /*
- * Copyright (C) 2011 Red Hat, Inc.
+ * Copyright © 2011 – 2017 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -25,7 +25,7 @@
 
 #define __GOA_BACKEND_INSIDE_GOA_BACKEND_H__
 #include <goabackend/goabackendenumtypes.h>
-#include <goabackend/goabackendtypes.h>
+#include <goabackend/goabackendenums.h>
 #include <goabackend/goaprovider.h>
 #undef __GOA_BACKEND_INSIDE_GOA_BACKEND_H__
 
