@@ -2,6 +2,7 @@
 /*
  * Copyright © 2023 GNOME Foundation Inc.
  * Contributor: Andy Holmes <andyholmes@gnome.org>
+ * Contributor: Jan-Michael Brummer <jan-michael.brummer1@volkswagen.de>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -20,65 +21,11 @@
 #include "config.h"
 
 #include <glib.h>
-#include <libsecret/secret.h>
+#include <gio/gio.h>
 
-
-static const SecretSchema oauth2_schema =
-{
-  .name = "org.gnome.OnlineAccounts.OAuth2",
-  .flags = SECRET_SCHEMA_NONE,
-  .attributes = {
-    {
-      .name = "goa-oauth2-client",
-      .type = SECRET_SCHEMA_ATTRIBUTE_STRING,
-    },
-    { "NULL", 0 }
-  }
-};
-
-static struct
-{
-  const char *client_id;
-}
-oauth2_providers[] =
-{
-#ifdef GOA_GOOGLE_ENABLED
-  {
-    .client_id = GOA_GOOGLE_CLIENT_ID,
-  },
-#endif
-#ifdef GOA_WINDOWS_LIVE_ENABLED
-  {
-    .client_id = GOA_WINDOWS_LIVE_CLIENT_ID,
-  },
-#endif
-#ifdef GOA_MS_GRAPH_ENABLED
-    {
-    .client_id = GOA_MS_GRAPH_CLIENT_ID,
-    },
-#endif
-  { NULL },
-};
-
-static gboolean
-get_oauth2_provider (const char  *needle,
-                     const char **client_out)
-{
-  g_return_val_if_fail (needle != NULL, FALSE);
-
-  for (unsigned int i = 0; oauth2_providers[i].client_id != NULL; i++)
-    {
-      if (g_str_equal (needle, oauth2_providers[i].client_id))
-        {
-          if (client_out)
-            *client_out = oauth2_providers[i].client_id;
-
-          return TRUE;
-        }
-    }
-
-  return FALSE;
-}
+#define OAUTH2_DBUS_HANDLER_NAME  "org.gnome.OnlineAccounts.OAuth2"
+#define OAUTH2_DBUS_HANDLER_PATH  "/org/gnome/OnlineAccounts/OAuth2"
+#define OAUTH2_DBUS_HANDLER_IFACE "org.gnome.OnlineAccounts.OAuth2"
 
 int
 main (int    argc,
@@ -88,8 +35,12 @@ main (int    argc,
   const char *scheme = NULL;
   const char *path = NULL;
   const char *client_id = NULL;
+  g_autoptr (GDBusConnection) connection = NULL;
+  g_autoptr (GVariant) reply = NULL;
   g_autoptr (GError) error = NULL;
 
+  g_set_prgname ("goa-oauth2-handler");
+
   if (argc < 2)
     {
       g_printerr ("%s: Missing URI\n", argv[0]);
@@ -107,10 +58,10 @@ main (int    argc,
    * See: https://developers.google.com/identity/protocols/oauth2/native-app
    */
   scheme = g_uri_get_scheme (uri);
-  if (scheme != NULL)
+  if (scheme != NULL && strstr (scheme, "."))
     {
       g_auto (GStrv) strv = g_strsplit (scheme, ".", -1);
-      g_autoptr (GString) tmp = g_string_new ("");
+      GString *tmp = g_string_new ("");
 
       for (unsigned int i = 0; strv[i] != NULL; i++)
         {
@@ -119,7 +70,7 @@ main (int    argc,
           g_string_prepend (tmp, strv[i]);
         }
 
-      get_oauth2_provider (tmp->str, &client_id);
+      client_id = g_string_free (tmp, FALSE);
     }
 
   /* Treat first path segment as client id */
@@ -141,18 +92,27 @@ main (int    argc,
       return EXIT_FAILURE;
     }
 
-  if (!secret_password_store_sync (&oauth2_schema,
-                                   SECRET_COLLECTION_SESSION,
-                                   "GNOME Online Accounts OAuth2 URI",
-                                   argv[1], /* Secret */
-                                   NULL,
-                                   &error,
-                                   "goa-oauth2-client", client_id,
-                                   NULL))
+  connection = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, &error);
+  if (connection == NULL)
     {
-      if (error != NULL)
-        g_printerr ("%s: Failed to store OAuth2 URI: %s\n", argv[0], error->message);
+      g_printerr ("Failed to get session bus: %s\n", error->message);
+      return EXIT_FAILURE;
+    }
 
+  reply = g_dbus_connection_call_sync (connection,
+                                       OAUTH2_DBUS_HANDLER_NAME,
+                                       OAUTH2_DBUS_HANDLER_PATH,
+                                       OAUTH2_DBUS_HANDLER_IFACE,
+                                       "Response",
+                                       g_variant_new ("(ss)", client_id, argv[1] /* redirect URI */),
+                                       NULL,
+                                       G_DBUS_CALL_FLAGS_NO_AUTO_START, /* not applicable */
+                                       -1,
+                                       NULL,
+                                       &error);
+  if (reply == NULL)
+    {
+      g_printerr ("Failed to forward authorization response: %s\n", error->message);
       return EXIT_FAILURE;
     }
 
